@@ -24,14 +24,11 @@
 use std::{
     collections::{BTreeMap, HashMap, HashSet, VecDeque},
     convert::TryFrom,
-    fs,
     future::Future,
-    io,
     ops::{Index, IndexMut},
     os::unix::fs::PermissionsExt,
     path::{Path, PathBuf},
     pin::Pin,
-    result,
     sync::{Arc, RwLock},
     time::Duration,
 };
@@ -52,7 +49,7 @@ use crate::command::actions::AccountAction;
 use crate::{
     conf::{data_types::SearchBackend, AccountConf, FileMailboxConf},
     jobs::{IsAsync, JobId, JoinHandle},
-    types::{ForkType, NotificationType, UIEvent},
+    types::{ForkedProcess, NotificationType, UIEvent},
     MainLoopHandler, StatusEvent, ThreadEvent,
 };
 
@@ -145,7 +142,7 @@ impl Drop for Account {
         if let Ok(data_dir) = xdg::BaseDirectories::with_profile("meli", self.name.as_ref()) {
             if let Ok(data) = data_dir.place_data_file("contacts") {
                 /* place result in cache directory */
-                let f = match fs::File::create(data) {
+                let f = match std::fs::File::create(data) {
                     Ok(f) => f,
                     Err(e) => {
                         eprintln!("{}", e);
@@ -157,7 +154,7 @@ impl Drop for Account {
 
                 permissions.set_mode(0o600); // Read/write for owner only.
                 f.set_permissions(permissions).unwrap();
-                let writer = io::BufWriter::new(f);
+                let writer = std::io::BufWriter::new(f);
                 if let Err(err) = serde_json::to_writer(writer, &self.contacts) {
                     eprintln!("{}", err);
                 };
@@ -224,8 +221,8 @@ impl Account {
 
         if let Ok(data) = data_dir.place_data_file("contacts") {
             if data.exists() {
-                let reader = io::BufReader::new(fs::File::open(data).unwrap());
-                let result: result::Result<Contacts, _> = serde_json::from_reader(reader);
+                let reader = std::io::BufReader::new(std::fs::File::open(data).unwrap());
+                let result: std::result::Result<Contacts, _> = serde_json::from_reader(reader);
                 if let Ok(data_t) = result {
                     for (id, c) in data_t.cards {
                         if !contacts.card_exists(id) && !c.external_resource() {
@@ -792,12 +789,13 @@ impl Account {
                     StatusEvent::DisplayMessage(format!("Running command {}", refresh_command)),
                 )));
             self.main_loop_handler
-                .send(ThreadEvent::UIEvent(UIEvent::Fork(ForkType::Generic {
-                    id: refresh_command.to_string().into(),
-                    command: Some(refresh_command.clone().into()),
-                    child,
-                })));
-            return Ok(());
+                .send(ThreadEvent::UIEvent(UIEvent::Fork(
+                    ForkedProcess::Generic {
+                        id: refresh_command.to_string().into(),
+                        command: Some(refresh_command.clone().into()),
+                        child,
+                    },
+                )));
         }
         let refresh_job = self.backend.write().unwrap().refresh(mailbox_hash);
         if let Ok(refresh_job) = refresh_job {
@@ -831,7 +829,7 @@ impl Account {
                         if let Some(wait) = wait {
                             sleep(wait).await;
                         }
-                        fut.await
+                        fut.into_future().await
                     };
                     let handle = self.main_loop_handler.job_executor.spawn(
                         "watch".into(),
@@ -891,7 +889,11 @@ impl Account {
         self.hash
     }
 
-    pub fn load(&mut self, mailbox_hash: MailboxHash, force: bool) -> result::Result<(), usize> {
+    pub fn load(
+        &mut self,
+        mailbox_hash: MailboxHash,
+        force: bool,
+    ) -> std::result::Result<(), usize> {
         if mailbox_hash.is_null() {
             return Err(0);
         }
@@ -912,16 +914,6 @@ impl Account {
             }
             MailboxStatus::None => {
                 if force && !self.active_jobs.values().any(|j| j.is_fetch(mailbox_hash)) {
-                    for ev in self.active_jobs.values().filter_map(|j| {
-                        if j.is_any_fetch() && !j.is_fetch(mailbox_hash) {
-                            j.cancel()
-                        } else {
-                            None
-                        }
-                    }) {
-                        self.main_loop_handler
-                            .send(ThreadEvent::UIEvent(UIEvent::StatusEvent(ev)));
-                    }
                     self.mailbox_entries
                         .entry(mailbox_hash)
                         .and_modify(|entry| {
@@ -1204,16 +1196,14 @@ impl Account {
         }
     }
 
+    #[inline]
     pub fn contains_key(&self, h: EnvelopeHash) -> bool {
         self.collection.contains_key(&h)
     }
-    pub fn operation(&self, h: EnvelopeHash) -> Result<Box<dyn BackendOp>> {
-        let operation = self.backend.read().unwrap().operation(h)?;
-        Ok(if self.settings.account.read_only {
-            ReadOnlyOp::new(operation)
-        } else {
-            operation
-        })
+
+    #[inline]
+    pub fn envelope_bytes_by_hash(&self, h: EnvelopeHash) -> ResultFuture<Vec<u8>> {
+        self.backend.read().unwrap().envelope_bytes_by_hash(h)
     }
 
     pub fn special_use_mailbox(&self, special_use: SpecialUsageMailbox) -> Option<MailboxHash> {
@@ -1224,12 +1214,13 @@ impl Account {
         ret.as_ref().map(|ret| ret.1.ref_mailbox.hash())
     }
 
-    /* Call only in Context::is_online, since only Context can launch the watcher
-     * threads if an account goes from offline to online. */
-    pub fn is_online(&mut self) -> Result<()> {
+    /// Callers should ensure `watch` is called if transitioning from
+    /// offline/error to online.
+    pub fn is_online(&mut self, force: bool) -> Result<()> {
         let (ret, wait) = match &mut self.is_online {
             IsOnline::Uninit => (Err(Error::new("Attempting connection.")), None),
-            IsOnline::True => return Ok(()),
+            IsOnline::True if !force => return Ok(()),
+            IsOnline::True => (Ok(()), None),
             IsOnline::Err {
                 value,
                 ref mut retries,
@@ -1256,7 +1247,7 @@ impl Account {
                 (ret, wait)
             }
         };
-        if !self.active_jobs.values().any(JobRequest::is_online) {
+        if force || !self.active_jobs.values().any(JobRequest::is_online) {
             let online_fut = self.backend.read().unwrap().is_online();
             if let Ok(online_fut) = online_fut {
                 let handle = match wait {
@@ -1586,7 +1577,7 @@ impl Account {
                                 .job_executor
                                 .set_job_success(job_id, false);
                             self.is_online.set_err(err);
-                            _ = self.is_online();
+                            _ = self.is_online(true);
                             self.main_loop_handler.send(ThreadEvent::UIEvent(
                                 UIEvent::AccountStatusChange(self.hash, None),
                             ));
@@ -1742,30 +1733,63 @@ impl Account {
                     }
                 }
                 JobRequest::Watch { ref mut handle } => {
-                    log::trace!("JobRequest::Watch finished??? ");
-                    if let Ok(Some(Err(err))) = handle.chan.try_recv() {
-                        if err.kind.is_timeout()
-                            || matches!(
-                                err.kind,
-                                ErrorKind::Network(NetworkErrorKind::HostLookupFailed)
-                            )
-                        {
-                            self.watch(Some(Duration::from_secs(3)));
-                        } else {
-                            self.main_loop_handler
-                                .job_executor
-                                .set_job_success(job_id, false);
-                            // [ref:TODO]: relaunch watch job with ratelimit for failure
-                            self.main_loop_handler.send(ThreadEvent::UIEvent(
-                                UIEvent::Notification {
-                                    title: Some(
-                                        format!("{}: watch thread failed", &self.name).into(),
-                                    ),
-                                    source: None,
-                                    body: err.to_string().into(),
-                                    kind: Some(NotificationType::Error(err.kind)),
-                                },
-                            ));
+                    log::trace!("JobRequest::Watch event");
+                    is_canceled! { handle };
+                    match handle.chan.try_recv() {
+                        Err(_) => { /* canceled */ }
+                        Ok(Some((None, _))) => {
+                            log::trace!("JobRequest::Watch stream returned None");
+                            self.watch(None);
+                            _ = self.is_online(true);
+                        }
+                        Ok(None) => {
+                            self.watch(None);
+                            _ = self.is_online(true);
+                        }
+                        Ok(Some((Some(ev), rest))) => {
+                            let handle = self.main_loop_handler.job_executor.spawn(
+                                "watch".into(),
+                                rest.into_future(),
+                                self.is_async(),
+                            );
+                            self.active_jobs
+                                .insert(handle.job_id, JobRequest::Watch { handle });
+                            match ev {
+                                Err(err) => {
+                                    log::trace!("JobRequest::Watch error {}", err);
+                                    if err.kind.is_timeout()
+                                        || matches!(
+                                            err.kind,
+                                            ErrorKind::Network(NetworkErrorKind::HostLookupFailed)
+                                        )
+                                        || err.is_recoverable()
+                                    {
+                                        self.watch(Some(Duration::from_secs(3)));
+                                        _ = self.is_online(true);
+                                    } else {
+                                        self.main_loop_handler
+                                            .job_executor
+                                            .set_job_success(job_id, false);
+                                        // [ref:TODO]: relaunch watch job with ratelimit for failure
+                                        self.main_loop_handler.send(ThreadEvent::UIEvent(
+                                            UIEvent::Notification {
+                                                title: Some(
+                                                    format!("{}: watch thread failed", &self.name)
+                                                        .into(),
+                                                ),
+                                                source: None,
+                                                body: err.to_string().into(),
+                                                kind: Some(NotificationType::Error(err.kind)),
+                                            },
+                                        ));
+                                    }
+                                }
+                                Ok(ev) => {
+                                    self.main_loop_handler.send(ThreadEvent::UIEvent(
+                                        UIEvent::BackendEvent(self.hash, ev),
+                                    ));
+                                }
+                            }
                         }
                     }
                 }
