@@ -71,6 +71,13 @@ pub use themes::*;
 
 pub use self::{composing::*, pgp::*, shortcuts::*, tags::*};
 
+/// Utility macro to access an [`AccountConf`] setting field from
+/// [`Context`](crate::Context) indexed by `$account_hash`
+///
+/// The value returned is the optionally overriden one in the
+/// [`AccountConf::conf_override`] field, otherwise the global one.
+///
+/// See also the [`mailbox_settings`](crate::mailbox_settings) macro.
 #[macro_export]
 macro_rules! account_settings {
     ($context:ident[$account_hash:expr].$setting:ident.$field:ident) => {{
@@ -87,6 +94,14 @@ macro_rules! account_settings {
     }};
 }
 
+/// Utility macro to access an [`AccountConf`] setting field from
+/// [`Context`](crate::Context) indexed by `$account_hash` and a mailbox.
+///
+/// The value returned is the optionally overriden one in the
+/// [`FileMailboxConf::conf_override`] field, otherwise the
+/// [`AccountConf::conf_override`] field, otherwise the global one.
+///
+/// See also the [`account_settings`] macro.
 #[macro_export]
 macro_rules! mailbox_settings {
     ($context:ident[$account_hash:expr][$mailbox_path:expr].$setting:ident.$field:ident) => {{
@@ -359,7 +374,9 @@ impl FileSettings {
 
         let mut stdout = std::io::stdout();
         let stdin = std::io::stdin();
-        crate::version_migrations::version_setup(&config_path, &mut stdout, &mut stdin.lock())?;
+        if !cfg!(test) {
+            crate::version_migrations::version_setup(&config_path, &mut stdout, &mut stdin.lock())?;
+        }
         Self::validate(config_path, false)
     }
 
