@@ -27,6 +27,7 @@ pub use dbus::*;
 use melib::{utils::datetime, UnixTimestamp};
 
 use super::*;
+use crate::conf::data_types::UINotifications;
 
 #[cfg(all(target_os = "linux", feature = "dbus-notifications"))]
 mod dbus {
@@ -177,13 +178,32 @@ mod dbus {
 /// Passes notifications to a user defined shell command
 #[derive(Debug, Default)]
 pub struct NotificationCommand {
+    /// Identifier of component.
     id: ComponentId,
 }
 
 impl NotificationCommand {
+    /// Create a new [`NotificationCommand`] component.
     pub fn new() -> Self {
         Self::default()
     }
+
+    /// Update an `xbiff` file at `path`.
+    ///
+    /// The ASCII byte `z` is appended to the file, unless the file size is
+    /// above 128 bytes in which case the file is truncated to 0 bytes.
+    pub fn update_xbiff(path: &str) -> Result<()> {
+        let mut file = std::fs::OpenOptions::new()
+            .append(true) /* writes will append to a file instead of overwriting previous contents */
+            .create(true) /* a new file will be created if the file does not yet already exist. */
+            .open(path)?;
+        if file.metadata()?.len() > 128 {
+            file.set_len(0)?;
+        } else {
+            std::io::Write::write_all(&mut file, b"z")?;
+        }
+        Ok(())
+    }
 }
 
 impl std::fmt::Display for NotificationCommand {
@@ -206,7 +226,7 @@ impl Component for NotificationCommand {
             if context.settings.notifications.enable {
                 if *kind == Some(NotificationType::NewMail) {
                     if let Some(ref path) = context.settings.notifications.xbiff_file_path {
-                        if let Err(err) = update_xbiff(path) {
+                        if let Err(err) = Self::update_xbiff(path) {
                             log::error!("Could not update xbiff file: {err}.");
                         }
                     }
@@ -234,7 +254,20 @@ impl Component for NotificationCommand {
                                 .children
                                 .entry(melib::identify!(NotificationCommand).into())
                                 .or_default()
-                                .push(child);
+                                .push(ForkedProcess::Generic {
+                                    id: melib::identify!(NotificationCommand).into(),
+                                    command: Some(
+                                        format!(
+                                            "{bin} {kind} {title} {body}",
+                                            kind = kind.map(|k| k.to_string()).unwrap_or_default(),
+                                            title =
+                                                title.as_ref().map(<_>::as_ref).unwrap_or("meli"),
+                                            body = body.as_ref()
+                                        )
+                                        .into(),
+                                    ),
+                                    child,
+                                });
                         }
                         Err(err) => {
                             log::error!("Could not run notification script: {err}.");
@@ -269,7 +302,13 @@ impl Component for NotificationCommand {
                                     .children
                                     .entry(melib::identify!(NotificationCommand).into())
                                     .or_default()
-                                    .push(child);
+                                    .push(ForkedProcess::Generic {
+                                        id: melib::identify!(NotificationCommand).into(),
+                                        command: Some(
+                                            format!("osascript -e {applescript}",).into(),
+                                        ),
+                                        child,
+                                    });
                                 return false;
                             }
                             Err(err) => {
@@ -304,19 +343,6 @@ impl Component for NotificationCommand {
     }
 }
 
-fn update_xbiff(path: &str) -> Result<()> {
-    let mut file = std::fs::OpenOptions::new()
-        .append(true) /* writes will append to a file instead of overwriting previous contents */
-        .create(true) /* a new file will be created if the file does not yet already exist. */
-        .open(path)?;
-    if file.metadata()?.len() > 128 {
-        file.set_len(0)?;
-    } else {
-        std::io::Write::write_all(&mut file, b"z")?;
-    }
-    Ok(())
-}
-
 #[derive(Debug)]
 /// On-screen-display messages.
 struct DisplayMessage {
@@ -328,17 +354,27 @@ struct DisplayMessage {
 #[derive(Debug)]
 /// Show notifications on [`Screen`].
 pub struct DisplayMessageBox {
+    /// Stored messages.
     messages: Vec<DisplayMessage>,
+    /// Timestamp of the latest time the widget was set to be displayed, used to
+    /// calculate expiration (i.e. when to stop displaying it).
     pub expiration_start: Option<UnixTimestamp>,
+    /// Whether the widget is to be displayed.
     pub active: bool,
+    /// Whether the widget needs to be redrawn.
     dirty: bool,
+    /// Whether the widget has been drawn at least once.
     pub initialised: bool,
+    /// Position of message from `messages` field to show.
     pub pos: usize,
+    /// The last [`Area`] this widget used to be drawn.
     cached_area: Area,
+    /// Identifier of component.
     id: ComponentId,
 }
 
 impl DisplayMessageBox {
+    /// Create a new [`DisplayMessageBox`] for this specific [`Screen`].
     pub fn new(sc: &Screen<Tty>) -> Self {
         Self {
             messages: Vec::new(),
@@ -352,21 +388,35 @@ impl DisplayMessageBox {
         }
     }
 
+    /// Returns the last [`Area`] this widget used to be drawn.
     #[inline]
     pub fn cached_area(&self) -> Area {
         self.cached_area
     }
 
+    /// Returns `true` if there are no messages.
     #[inline]
     pub fn is_empty(&self) -> bool {
         self.messages.is_empty()
     }
 
+    /// Returns the amount of messages.
     #[inline]
     pub fn len(&self) -> usize {
         self.messages.len()
     }
 
+    /// Prevent box from being shown unless armed again.
+    #[inline]
+    pub fn deactivate(&mut self) {
+        self.active = false;
+        self.set_dirty(true);
+        self.initialised = false;
+        self.expiration_start = None;
+    }
+
+    /// Append a message if its contents are not equal to the last existing
+    /// message.
     #[inline]
     pub fn try_push(&mut self, msg: String, timestamp: UnixTimestamp) -> bool {
         if let Some(ref mut last) = self.messages.last_mut().filter(|el| el.msg == msg) {
@@ -380,6 +430,52 @@ impl DisplayMessageBox {
         });
         true
     }
+
+    /// Trigger display of last message, if any.
+    #[inline]
+    pub fn arm(&mut self, context: &mut Context) {
+        if self.messages.is_empty() {
+            return;
+        }
+        self.pos = self.messages.len().saturating_sub(1);
+        match context.settings.notifications.ui_notifications {
+            UINotifications::Show => {
+                self.active = true;
+                self.initialised = false;
+                self.set_dirty(true);
+                self.expiration_start = None;
+            }
+            UINotifications::Hide => {}
+            UINotifications::System => {
+                context.replies.push_back(UIEvent::Notification {
+                    title: None,
+                    source: None,
+                    body: self.messages[self.pos].msg.clone().into(),
+                    kind: Some(NotificationType::Info),
+                });
+            }
+        }
+    }
+
+    /// Activate and set position to previous message, if any.
+    #[inline]
+    pub fn show_previous(&mut self) {
+        self.expiration_start = Some(datetime::now());
+        self.active = true;
+        self.initialised = false;
+        self.set_dirty(true);
+        self.pos = self.pos.saturating_sub(1);
+    }
+
+    /// Activate and set position to next message, if any.
+    #[inline]
+    pub fn show_next(&mut self) {
+        self.expiration_start = Some(datetime::now());
+        self.active = true;
+        self.initialised = false;
+        self.set_dirty(true);
+        self.pos = self.len().saturating_sub(1).min(self.pos + 1);
+    }
 }
 
 impl std::fmt::Display for DisplayMessageBox {
