@@ -19,23 +19,10 @@
  * along with meli. If not, see <http://www.gnu.org/licenses/>.
  */
 
-use crate::utils::random::{clock, random_u64};
-
-fn base36(mut m: u64) -> String {
-    let mut stack = Vec::with_capacity(32);
-
-    while m >= 36 {
-        stack.push((m % 36) as u32);
-        m /= 36;
-    }
-
-    let mut ret = String::with_capacity(stack.len());
-
-    while let Some(d) = stack.pop() {
-        ret.push(char::from_digit(d, 36).unwrap());
-    }
-    ret
-}
+use crate::utils::{
+    base36,
+    random::{clock, random_u64},
+};
 
 pub fn gen_message_id(fqdn: &str) -> String {
     let clock = base36(clock());
