@@ -108,6 +108,7 @@ impl ErrorKind {
     is_variant! { is_from_linked_library, LinkedLibrary(_) }
     is_variant! { is_network, Network(_) }
     is_variant! { is_network_down, Network(ref k) if k.is_network_down() }
+    is_variant! { is_disconnected, Network(ref k) if k.is_disconnected() }
     is_variant! { is_not_implemented, NotImplemented }
     is_variant! { is_not_supported, NotSupported }
     is_variant! { is_not_found, NotFound }
@@ -398,7 +399,11 @@ impl std::fmt::Display for ErrorChainDisplay<'_> {
                 other => write!(fmt, "{other}: ")?,
             }
             if let Some(ref inner) = cur.current.inner {
-                write!(fmt, "{}", inner)?;
+                if cur.current.summary.is_empty() {
+                    write!(fmt, "{}", inner)?;
+                } else {
+                    write!(fmt, "{} ({})", inner, cur.current.summary)?;
+                };
             } else {
                 write!(fmt, "{}", cur.current.summary)?;
                 if let Some(details) = cur.current.details.as_ref() {
@@ -455,6 +460,16 @@ impl From<io::Error> for Error {
         let s = err.to_string();
         let kind = if s.contains("failed to lookup address information") {
             ErrorKind::Network(NetworkErrorKind::HostLookupFailed)
+        } else if s.contains("the SSL session has been shut down") {
+            // FIXME: This is an OS-specific error from the `openssl` crate, make the check
+            // more intelligent than a hardcoded string `contains`.
+            // <https://github.com/sfackler/rust-openssl/blob/538a5cb737e8d83085553cac01643820dc7ff205/openssl/src/ssl/error.rs#L100-L123>
+            ErrorKind::Network(NetworkErrorKind::TLSConnectionFailed)
+        } else if s.contains("OpenSSL error") {
+            // FIXME: This is an OS-specific error from the `openssl` crate, make the check
+            // more intelligent than a hardcoded string `contains`.
+            // <https://github.com/sfackler/rust-openssl/blob/538a5cb737e8d83085553cac01643820dc7ff205/openssl/src/ssl/error.rs#L100-L123>
+            ErrorKind::Network(NetworkErrorKind::InvalidTLSConnection)
         } else if let Some(errno) = err.raw_os_error() {
             ErrorKind::OSError(Errno::from_raw(errno))
         } else {
