@@ -228,6 +228,7 @@ impl MailBackend for NntpType {
     }
 
     fn fetch(&mut self, mailbox_hash: MailboxHash) -> ResultStream<Vec<Envelope>> {
+        let is_online_fut = self.is_online()?;
         let mut state = FetchState {
             mailbox_hash,
             uid_store: self.uid_store.clone(),
@@ -235,15 +236,21 @@ impl MailBackend for NntpType {
             total_low_high: None,
         };
         Ok(Box::pin(try_fn_stream(|emitter| async move {
+            is_online_fut.await?;
             {
                 let f = &state.uid_store.mailboxes.lock().await[&state.mailbox_hash];
                 f.exists.lock().unwrap().clear();
                 f.unseen.lock().unwrap().clear();
             };
             loop {
-                if let Some(ret) = state.fetch_envs().await? {
-                    emitter.emit(ret).await;
-                    continue;
+                match state.fetch_envs().await {
+                    Err(err) if err.kind.is_disconnected() => continue,
+                    Err(err) => return Err(err),
+                    Ok(Some(ret)) => {
+                        emitter.emit(ret).await;
+                        continue;
+                    }
+                    Ok(None) => {}
                 }
                 break;
             }
@@ -252,12 +259,14 @@ impl MailBackend for NntpType {
     }
 
     fn refresh(&mut self, mailbox_hash: MailboxHash) -> ResultFuture<()> {
+        let is_online_fut = self.is_online()?;
         let uid_store = self.uid_store.clone();
         let connection = self.connection.clone();
         let timeout_dur = self.server_conf.timeout_dur;
         Ok(Box::pin(async move {
-            /* To get updates, either issue NEWNEWS if it's supported by the server, and
-             * fallback to OVER otherwise */
+            is_online_fut.await?;
+            // To get updates, either issue NEWNEWS if it's supported by the server, and
+            // fallback to OVER otherwise
             let mbox: NntpMailbox = uid_store
                 .mailboxes
                 .lock()
@@ -366,10 +375,12 @@ impl MailBackend for NntpType {
     }
 
     fn mailboxes(&self) -> ResultFuture<HashMap<MailboxHash, Mailbox>> {
+        let is_online_fut = self.is_online()?;
         let uid_store = self.uid_store.clone();
         let connection = self.connection.clone();
         let timeout_dur = self.server_conf.timeout_dur;
         Ok(Box::pin(async move {
+            is_online_fut.await?;
             Self::nntp_mailboxes(&connection, timeout_dur).await?;
             let mailboxes_lck = uid_store.mailboxes.lock().await;
             let ret = mailboxes_lck
@@ -385,7 +396,6 @@ impl MailBackend for NntpType {
         let timeout_dur = self.server_conf.timeout_dur;
         Ok(Box::pin(async move {
             let mut conn = timeout(timeout_dur, connection.lock()).await?;
-            log::trace!("is_online");
             match timeout(timeout_dur, conn.connect()).await {
                 Ok(Ok(())) => Ok(()),
                 Err(err) | Ok(Err(err)) => {
@@ -396,16 +406,17 @@ impl MailBackend for NntpType {
         }))
     }
 
-    fn watch(&self) -> ResultFuture<()> {
+    fn watch(&self) -> ResultStream<BackendEvent> {
         Err(
-            Error::new("Watching is currently uniplemented for nntp backend")
+            Error::new("Watching is currently unimplemented for nntp backend")
                 .set_kind(ErrorKind::NotImplemented),
         )
     }
 
-    fn operation(&self, env_hash: EnvelopeHash) -> Result<Box<dyn BackendOp>> {
+    fn envelope_bytes_by_hash(&self, hash: EnvelopeHash) -> ResultFuture<Vec<u8>> {
+        let is_online_fut = self.is_online()?;
         let (uid, mailbox_hash) =
-            if let Some(v) = self.uid_store.hash_index.lock().unwrap().get(&env_hash) {
+            if let Some(v) = self.uid_store.hash_index.lock().unwrap().get(&hash) {
                 *v
             } else {
                 return Err(Error::new(
@@ -413,12 +424,17 @@ impl MailBackend for NntpType {
                      requested it.",
                 ));
             };
-        Ok(Box::new(NntpOp::new(
+        let op = NntpOp::new(
             uid,
             mailbox_hash,
             self.connection.clone(),
             self.uid_store.clone(),
-        )))
+        );
+
+        Ok(Box::pin(async move {
+            is_online_fut.await?;
+            op.as_bytes().await
+        }))
     }
 
     fn save(
@@ -427,7 +443,7 @@ impl MailBackend for NntpType {
         _mailbox_hash: MailboxHash,
         _flags: Option<Flag>,
     ) -> ResultFuture<()> {
-        Err(Error::new("NNTP doesn't support saving."))
+        Err(Error::new("NNTP doesn't support saving.").set_kind(ErrorKind::NotSupported))
     }
 
     fn copy_messages(
@@ -437,7 +453,7 @@ impl MailBackend for NntpType {
         _destination_mailbox_hash: MailboxHash,
         _move_: bool,
     ) -> ResultFuture<()> {
-        Err(Error::new("NNTP doesn't support copying/moving."))
+        Err(Error::new("NNTP doesn't support copying/moving.").set_kind(ErrorKind::NotSupported))
     }
 
     fn set_flags(
@@ -502,7 +518,7 @@ impl MailBackend for NntpType {
         _env_hashes: EnvelopeHashBatch,
         _mailbox_hash: MailboxHash,
     ) -> ResultFuture<()> {
-        Err(Error::new("NNTP doesn't support deletion."))
+        Err(Error::new("NNTP doesn't support deletion.").set_kind(ErrorKind::NotSupported))
     }
 
     fn as_any(&self) -> &dyn std::any::Any {
@@ -523,7 +539,7 @@ impl MailBackend for NntpType {
     ) -> ResultFuture<(MailboxHash, HashMap<MailboxHash, Mailbox>)> {
         Err(
             Error::new("Creating mailbox is not supported for nntp backend.")
-                .set_kind(ErrorKind::NotImplemented),
+                .set_kind(ErrorKind::NotSupported),
         )
     }
 
@@ -554,7 +570,7 @@ impl MailBackend for NntpType {
         _new_path: String,
     ) -> ResultFuture<Mailbox> {
         Err(
-            Error::new("Renaming mailbox is not supported for nntp backend.")
+            Error::new("Renaming a mailbox is not supported for nntp backend.")
                 .set_kind(ErrorKind::NotSupported),
         )
     }
@@ -585,13 +601,17 @@ impl MailBackend for NntpType {
         mailbox_hash: Option<MailboxHash>,
         _flags: Option<Flag>,
     ) -> ResultFuture<()> {
+        let is_online_fut = self.is_online()?;
         let connection = self.connection.clone();
         let timeout_dur = self.server_conf.timeout_dur;
         Ok(Box::pin(async move {
+            is_online_fut.await?;
             let mut conn = timeout(timeout_dur, connection.lock()).await?;
             let stream = conn.stream.as_ref()?;
             if !stream.supports_submission {
-                return Err(Error::new("Server prohibits posting."));
+                return Err(
+                    Error::new("Server prohibits posting.").set_kind(ErrorKind::NotSupported)
+                );
             }
             // [ref:TODO] normalize CRLF in `bytes`
             let mut res = String::with_capacity(8 * 1024);
@@ -764,7 +784,7 @@ impl NntpType {
         while !mailboxes.is_empty() {
             let mut command = "LIST ACTIVE ".to_string();
             'batch: while let Some(m) = mailboxes.pop() {
-                /* first check if the group name itself is too big for `LIST ACTIVE`. */
+                // first check if the group name itself is too big for `LIST ACTIVE`.
                 if "LIST ACTIVE ".len() + m.len() + "\r\n".len() >= 512 {
                     log::warn!(
                         "{}: Newsgroup named {} has a name that exceeds RFC 3977 limits of \
@@ -1008,23 +1028,26 @@ impl FetchState {
 
         if total_low_high.is_none() {
             conn.select_group(mailbox_hash, true, &mut res).await?;
-            /*
-             *   Parameters
-                     group     Name of newsgroup
-                     number    Estimated number of articles in the group
-                     low       Reported low water mark
-                     high      Reported high water mark
-            */
+            // Parameters
+            //     group     Name of newsgroup
+            //     number    Estimated number of articles in the group
+            //     low       Reported low water mark
+            //     high      Reported high water mark
             let s = res.split_whitespace().collect::<SmallVec<[&str; 6]>>();
             let path = conn.uid_store.mailboxes.lock().await[&mailbox_hash]
                 .name()
                 .to_string();
             if s.len() != 5 {
-                return Err(Error::new(format!(
+                let err = Error::new(format!(
                     "{} Could not select newsgroup {}: expected GROUP response but got: {}",
                     &uid_store.account_name, path, res
                 ))
-                .set_kind(ErrorKind::ProtocolError));
+                .set_kind(ErrorKind::ProtocolError);
+                conn.stream = Err(err.clone());
+                conn.connect().await?;
+                // If .connect() didn't error out, this means our error was a connection
+                // failure.
+                return Err(err.set_kind(ErrorKind::Network(NetworkErrorKind::ConnectionFailed)));
             }
             let total = usize::from_str(s[1]).unwrap_or(0);
             let low = usize::from_str(s[2]).unwrap_or(0);
